@@ -38,6 +38,7 @@ fi
 . $MAINDIR/build_setup.conf
 . $MAINDIR/scripts/helpers.sh
 . $MAINDIR/scripts/packethandlers.sh
+. $MAINDIR/scripts/buildfuncs.sh
 
 if ! log_init
 then
@@ -50,6 +51,12 @@ then
   exit 1
 fi
 
+if test $(id -u) == 0
+then
+  log_error "Do not run crosser.sh as root. That can destroy your system."
+  exit 1
+fi
+
 if test "x$1" != "x" ; then
   SETUP="$1"
 else
@@ -66,7 +73,7 @@ else
 
     while test "x$ANSWER" != "xyes" && test "x$ANSWER" != "xno"
     do
-      echo "You gave no steps. Building everything from scratch will take hours."
+      echo "You gave no steps. Building everything from scratch will take very long time."
       echo "Are you sure you want to continue?"
       echo "yes/no"
       echo -n "> "
@@ -84,7 +91,7 @@ else
     fi
   fi
   log_write 1 "No build steps defined, building everything"
-  STEPPARAM="native:gtk"
+  STEPPARAM="native:sdl"
 fi
 
 . $MAINDIR/steps/stepfuncs.sh
@@ -134,7 +141,7 @@ else
   BUILD="$NATIVE_ARCH-$NATIVE_VENDOR-$NATIVE_OS"
 fi
 
-NATIVE_PREFIX=$(setup_prefix_default "/usr/local/crosser/$CROSSER_VERSION/host" "$NATIVE_PREFIX")
+NATIVE_PREFIX=$(setup_prefix_default "$HOME/.crosser/$CROSSER_VERSION/host" "$NATIVE_PREFIX")
 
 CH_ERROR="$(check_crosser_env $NATIVE_PREFIX native)"
 if test "x$CH_ERROR" != "x" && test "x$STEP_NATIVE" != "xyes" ; then
@@ -146,9 +153,9 @@ fi
 # If native build, and cross-compile is not already being forced
 if test "x$TARGET" = "x$BUILD" && test "x$CROSS_OFF" = "x" ; then
   CROSS_OFF=yes
-  DEFPREFIX="/usr/local/crosser/$CROSSER_VERSION/native"
+  DEFPREFIX="$HOME/.crosser/$CROSSER_VERSION/native"
 else
-  DEFPREFIX="/usr/local/crosser/$CROSSER_VERSION/$TARGET-$LIBC_MODE"
+  DEFPREFIX="$HOME/.crosser/$CROSSER_VERSION/$TARGET-$LIBC_MODE"
 fi
 
 PREFIX=$(setup_prefix_default "$DEFPREFIX" "$PREFIX")
@@ -161,7 +168,8 @@ then
   if test "x$CH_ERROR" != "x" &&
      test "x$STEP_CHAIN" != "xyes" &&
      ( test "x$STEP_BASELIB" = "xyes" ||
-       test "x$STEP_GTK" = "xyes" )
+       test "x$STEP_GTK" = "xyes" ||
+       test "x$STEP_SDL" = "xyes" )
   then
     log_error "$CH_ERROR"
     log_error "Step 'chain' for building environment is not enabled."
@@ -171,165 +179,6 @@ fi
 
 #############################################################################
 
-# Generic function to compile one component
-#
-# $1   - Build directory in build hierarchy
-# $2   - Sourcedir in source hierarchy
-# $3   - Configure options
-# $4   - Make targets
-# [$5] - Build in srcdir ('yes')
-build_generic() {
-  log_packet "$1"
-
-  if test "x$5" != "xyes"
-  then
-    if test -d "$MAINBUILDDIR/$1"
-    then
-      rm -Rf "$MAINBUILDDIR/$1"
-    fi
-
-    if ! mkdir -p $MAINBUILDDIR/$1
-    then
-       log_error "Failed to create directory \"$MAINBUILDDIR/$1\""
-       return 1
-    fi
-    if ! cd $MAINBUILDDIR/$1
-    then
-       log_error "Failed to change workdir to \"$MAINBUILDDIR/$1\""
-       return 1
-    fi
-  else
-    if ! cd $MAINSRCDIR/$2
-    then
-       log_error "Failed to change workdir to \"$MAINSRCDIR/$2\""
-       return 1
-    fi
-  fi
-
-  log_write 1 "Configuring: $2"
-  log_write 3 "  Options: \"$3\""
-  log_flags
-
-  if ! "$MAINSRCDIR/$2/configure" $3 \
-      2>>$MAINLOGDIR/stderr.log >>$MAINLOGDIR/stdout.log
-  then
-    log_error "Configure failed: $1"
-    return 1
-  fi
-
-  if test "x$4" != "x-" ; then
-    if test "x$4" != "x" ; then
-      MKTARGETS="$4"
-    else
-      MKTARGETS="all install"
-    fi
-
-    # We need old, dummy, libc.so for running configure for glibc itself,
-    # but it has to be removed before we make real one.
-    if test "x$1" = "xtgt-glibc" && test -e "$SYSPREFIX/usr/lib/libc.so"
-    then
-      log_write 3 "  Removing old libc.so"
-      rm -f "$SYSPREFIX/usr/lib/libc.so"
-    fi
-
-    if test "x$CROSSER_CORES" != "x"
-    then
-       COREOPT="-j $CROSSER_CORES"
-    fi
-
-    log_write 1 "Building $2"
-    log_write 3 "  Make targets: $MKTARGETS"
-    if ! make $COREOPT $MKTARGETS \
-        2>>$MAINLOGDIR/stderr.log >>$MAINLOGDIR/stdout.log
-    then
-      log_error "Make failed: $1"
-      return 1
-    fi
-  fi
-
-  hash -r
-
-  return 0
-}
-
-# Build with compiler built in native step
-#
-# $1 - Component to compile
-# $2 - Sourcedir in source hierarchy
-# $3 - Configure options
-# $4 - Make targets
-build_with_native_compiler() {
-  CONFOPTIONS="--build=$BUILD --host=$BUILD --target=$TARGET --prefix=$CROSSPREFIX $3 --disable-nls"
-
-  export CFLAGS="-O2"
-  export CPPFLAGS=""
-  export LDFLAGS=""
-
-  if ! build_generic "cross-$1" "$2" "$CONFOPTIONS" "$4"
-  then
-    return 1
-  fi
-}
-
-# Build component using cross-compiler
-#
-# $1   - Component name
-# $2   - Source dir in source hierarchy
-# $3   - Configure options
-# $4   - Make targets
-# [$5] - Destdir
-build_with_cross_compiler() {
-  if test "x$3" = "x"
-  then
-    PREFIXOPTION="--prefix=/usr"
-  else
-    PREFIXOPTION=""
-  fi
-  CONFOPTIONS="--build=$BUILD --host=$TARGET --target=$TARGET $PREFIXOPTION $3 --disable-nls"
-
-  export CFLAGS="-O2"
-  export CPPFLAGS="-isystem $SYSPREFIX/include -isystem $SYSPREFIX/usr/include"
-  export LDFLAGS="-L$SYSPREFIX/lib -L$SYSPREFIX/usr/lib"
-
-  if test "x$4" = "x"
-  then
-    MAKETARGETS="all install"
-  else
-    MAKETARGETS="$4"
-  fi
-
-  if test "x$5" = "x"
-  then
-    DESTDIR="$SYSPREFIX"
-  else
-    DESTDIR="$5"
-  fi
-
-  if ! build_generic "tgt-$1" "$2" "$CONFOPTIONS" "DESTDIR=$DESTDIR $MAKETARGETS"
-  then
-    return 1
-  fi
-}
-
-# Build component to native directory hierarchy
-#
-# $1   - Component name
-# $2   - Source dir in source hierarchy
-# $3   - Configure options
-# $4   - Make targets
-build_for_host() {
-  CONFOPTIONS="--build=$BUILD --host=$BUILD --target=$BUILD --prefix=$NATIVE_PREFIX $3"
-
-  export CFLAGS="-march=native -O2"
-  export CPPFLAGS=""
-  export LDFLAGS="-Wl,-rpath,$NATIVE_PREFIX/lib -L$NATIVE_PREFIX/lib"
-
-  if ! build_generic "host-$1" "$2" "$CONFOPTIONS" "$4"
-  then
-    return 1
-  fi
-}
-
 # Build glibc using cross-compiler
 #
 # $1   - Component to compile
@@ -408,8 +257,7 @@ create_host_dirs()
     return 1
   fi
 
-  if ! mkdir -p $NATIVE_PREFIX/crosser ||
-     ! mkdir -p $NATIVE_PREFIX/usr/include
+  if ! mkdir -p $NATIVE_PREFIX/usr/include
   then
     log_error "Failed to create host directories under \"$NATIVE_PREFIX\""
     return 1
@@ -420,8 +268,7 @@ create_host_dirs()
 #
 create_target_dirs()
 {
-  if ! mkdir -p $PREFIX/crosser        ||
-     ! mkdir -p $SYSPREFIX/etc         ||
+  if ! mkdir -p $SYSPREFIX/etc         ||
      ! mkdir -p $SYSPREFIX/include     ||
      ! mkdir -p $SYSPREFIX/usr/include ||
      ! mkdir -p $SYSPREFIX/usr/lib
@@ -431,13 +278,23 @@ create_target_dirs()
   fi
 }
 
-# Install kernel headers
+# Install kernel or kernel headers
 #
-kernel_header_setup() {
-  log_write 1 "Kernel setup"
-  log_packet "Kernel headers"
+# [$1] - "full" - build kernel. Default is to only install headers
+#
+kernel_setup() {
+  if test "x$1" = "xfull"
+  then
+    log_write 1 "Kernel setup"
+    log_packet "Kernel"
+  else
+    log_write 1 "Kernel header setup"
+    log_packet "Kernel headers"
+  fi
 
-  if ! unpack_component linux $VERSION_KERNEL
+  # Kernel may have been unpacked already in chain step
+  if ! test -d $MAINSRCDIR/linux-$VERSION_KERNEL &&
+     ! unpack_component linux $VERSION_KERNEL
   then
     log_error "Failed to unpack kernel"
     return 1
@@ -472,13 +329,25 @@ kernel_header_setup() {
       return 1
     fi
 
-    MAKEPARAMS="$CROSSPARAM $KERN_PARAM INSTALL_HDR_PATH=$SYSPREFIX/usr headers_install"
+    if test "x$1" = "xfull"
+    then
+      MAKETARGETS="bzImage modules modules_install"
+      log_write 1 "Building linux kernel"
+    else
+      MAKETARGETS="headers_install"
+    fi
+    MAKEPARAMS="$CROSSPARAM $KERN_PARAM INSTALL_HDR_PATH=$SYSPREFIX/usr INSTALL_MOD_PATH=$SYSPREFIX $MAKETARGETS"
 
     log_write 3 "  Make params: $MAKEPARAMS"
     if ! make $MAKEPARAMS \
                 2>>$MAINLOGDIR/stderr.log >>$MAINLOGDIR/stdout.log
     then
-      log_error "Kernel headers install failed"
+      if test "x$1" = "xfull"
+      then
+        log_error "Kernel build failed"
+      else
+        log_error "Kernel headers install failed"
+      fi
       return 1
     fi
   ) then
@@ -491,16 +360,27 @@ kernel_header_setup() {
 # $1 - Command to link
 # $2 - Required ('no')
 link_host_command() {
+  CMDDIRS="/usr/local/sbin /usr/sbin /sbin"
   CMDPATH="$(which $1)"
 
   if test "x$CMDPATH" = "x"
   then
-    if test "x$2" != "xno"
+    for CMDDIR in $CMDDIRS
+    do
+      if test "x$CMDPATH" = "x" && test -x "$CMDDIR/$1"
+      then
+        CMDPATH="$CMDDIR/$1"
+      fi
+    done
+    if test "x$CMDPATH" = "x"
     then
-      log_error "Cannot find host command $1"
-      return 1
-    else
-      return 0
+      if test "x$2" != "xno"
+      then
+        log_error "Cannot find host command $1"
+        return 1
+      else
+        return 0
+      fi
     fi
   fi
 
@@ -517,10 +397,12 @@ link_host_command() {
 # 1 - Failure
 setup_host_commands() {
   # Absolutely required commands
-  HOST_COMMANDS_REQ="mkdir touch true false chmod rm which sed grep expr cat echo sort mv cp ln cmp test comm ls rmdir tr date uniq sleep diff basename dirname tail head env uname cut readlink od egrep fgrep wc make find pwd tar m4 awk getconf expand perl bison bzip2 flex makeinfo wget pod2man msgfmt pkg-config sh glib-genmarshal"
+  HOST_COMMANDS_REQ="mkdir touch true false chmod rm which sed grep expr cat echo sort mv cp ln cmp test comm ls rmdir tr date uniq sleep diff basename dirname tail head env uname cut readlink od egrep fgrep wc make find pwd tar m4 awk getconf expand perl bison bzip2 flex makeinfo install whoami depmod wget pod2man msgfmt pkg-config sh glib-genmarshal hostname dnsdomainname mktemp"
   # Usefull commands
   HOST_COMMANDS_TRY="dpkg-source md5sum gpg sha1sum sha256sum gzip gunzip patch"
 
+  log_write 1 "Setting up hostbin"
+
   if ! mkdir -p $NATIVE_PREFIX/hostbin ; then
     log_error "Cannot create directory $PREFIX/hostbin"
     return 1
@@ -546,7 +428,7 @@ setup_host_commands() {
 # Build dummy glibc objects
 #
 dummy_glibc_objects() {
-  log_write 1 "Generating dummy glibc objects"
+  log_write 1 "Generating dummy c-lib objects"
 
   if ! mkdir $MAINBUILDDIR/crt ; then
      return 1
@@ -561,7 +443,9 @@ dummy_glibc_objects() {
        log_error "crt.o build failed"
        return 1
     fi
-    if ! $TARGET-gcc -c -shared -fPIC $MAINDIR/scripts/dummyclib.c -o libc.so ; then
+    if ! $TARGET-gcc -c -shared -fPIC $MAINDIR/scripts/dummyclib.c -o libc.so \
+         2>>$MAINLOGDIR/stderr.log >>$MAINLOGDIR/stdout.log
+    then
         log_error "Failed to build dummy libc.so"
        return 1
     fi
@@ -595,10 +479,13 @@ dummy_glibc_objects() {
 # Prepare binutils source tree
 #
 prepare_binutils_src() {
-  if ! unpack_component binutils     $VERSION_BINUTILS            ||
+  if ! unpack_component binutils     $VERSION_BINUTILS              ||
      ! ( is_greater_version $VERSION_BINUTILS 2.18     ||
         (  patch_src binutils-$VERSION_BINUTILS binutils_makeinfo &&
-           cd $MAINSRCDIR/binutils-$VERSION_BINUTILS && autoconf ))
+           cd $MAINSRCDIR/binutils-$VERSION_BINUTILS && autoconf )) ||
+     ! ( is_greater_version $VERSION_BINUTILS 2.18     ||
+        (  patch_src binutils-$VERSION_BINUTILS binutils_printffix &&
+	   patch_src binutils-$VERSION_BINUTILS binutils_ignret ))
   then
     log_error "Binutils setup failed"
     return 1
@@ -631,29 +518,6 @@ prepare_gcc_src() {
   fi
 }
 
-# Upack glibc tarballs. They have to be extracted from deb-packet already
-#
-unpack_glibc() {
-  if is_minimum_version $VERSION_GLIBC 2.8
-  then
-    ln -s glibc $MAINSRCDIR/glibc-$VERSION_GLIBC
-    return 0
-  fi
-
-  GPACKNAME=$(ls -1 $MAINSRCDIR/glibc/glibc-$VERSION_GLIBC*.tar.bz2)
-  GPPACKNAME=$(ls -1 $MAINSRCDIR/glibc/glibc-ports-$VERSION_GLIBC*.tar.bz2)
-  LTPACKNAME=$(ls -1 $MAINSRCDIR/glibc/glibc-linuxthreads-*.tar.bz2)
-
-  if ! tar xjf $GPACKNAME -C $MAINSRCDIR ||
-     ! tar xjf $GPPACKNAME -C $MAINSRCDIR/glibc-$VERSION_GLIBC ||
-     ! tar xjf $LTPACKNAME -C $MAINSRCDIR/glibc-$VERSION_GLIBC ||
-     ! ln -s glibc-ports-$VERSION_GLIBC $MAINSRCDIR/glibc-$VERSION_GLIBC/ports
-  then
-    log_error "Unpacking glibc tarballs failed"
-    return 1
-  fi
-}
-
 if test "x$TARGET" = "x$BUILD" && test "x$CROSS_OFF" = "x"
 then
   CROSS_OFF=yes
@@ -705,14 +569,15 @@ then
   fi
 fi
 
-LDCONFIG=/sbin/ldconfig
-
 if test "x$STEP_NATIVE" = "xyes" ; then
   STEP="native"
   STEPADD="  "
 
   BASEVER_LIBTOOL="$(basever_libtool $VERSION_LIBTOOL)"
 
+  # Build of latter tools uses earlier tools
+  export PATH=$NATIVE_PREFIX/bin:$PATH
+
   if ! create_host_dirs     ||
      ! unpack_component libtool  $VERSION_LIBTOOL             ||
      ! build_for_host   libtool  libtool-$BASEVER_LIBTOOL     ||
@@ -745,8 +610,7 @@ if test "x$STEP_NATIVE" = "xyes" ; then
     exit 1
   fi
 
-  echo "Setup:   Native"           >  "$NATIVE_PREFIX/crosser/crosser.hierarchy"
-  echo "Version: $CROSSER_VERSION" >> "$NATIVE_PREFIX/crosser/crosser.hierarchy"
+  write_crosser_env "$NATIVE_PREFIX" Native
 fi
 
 # None of these path variables contain original $PATH.
@@ -788,7 +652,7 @@ then
     hash -r
     # Prepare kernel sources while we are still using compiler with
     # original sysroot
-    if ! kernel_header_setup
+    if ! kernel_setup
     then
       crosser_error "Kernel header setup failed"
       exit 1
@@ -858,41 +722,55 @@ then
       exit 1
     fi
 
-    if test "x$BUILD" != "x$TARGET" && ! kernel_header_setup
+    if test "x$BUILD" != "x$TARGET" && ! kernel_setup
     then
       crosser_error "Kernel header setup failed"
       exit
     fi
 
-    if ! unpack_component glibc $VERSION_GLIBC_DEB                        ||
-       ! unpack_glibc                                                     ||
-       ! (is_minimum_version $VERSION_GLIBC 2.8 ||
-          patch_src glibc-$VERSION_GLIBC glibc_upstream_finc)             ||
-       ! patch_src glibc-$VERSION_GLIBC glibc_nomanual                    ||
-       ! (is_minimum_version $VERSION_GLIBC 2.8 ||
-          patch_src glibc-$VERSION_GLIBC/glibc-ports-$VERSION_GLIBC glibc_ports_arm_docargs) ||
-       ! (is_minimum_version $VERSION_GLIBC 2.8 ||
-          patch_src glibc-$VERSION_GLIBC/glibc-ports-$VERSION_GLIBC glibc_ports_arm_pageh_inc) ||
-       ! patch_src glibc-$VERSION_GLIBC/ports glibc_ports_arm_tlsinc      ||
-       ! (is_smaller_version $VERSION_GLIBC 2.9 ||
-          patch_src glibc-$VERSION_GLIBC/ports glibc_upstream_arm_sigsetjmp)
+    LIBCNAME=glibc
+    LIBCVER=$VERSION_GLIBC
+    LIBCDIR=$LIBCNAME-$LIBCVER
+
+    if ! unpack_component $LIBCNAME       $LIBCVER          ||
+       ! unpack_component $LIBCNAME-ports $LIBCVER $LIBCDIR
     then
-      crosser_error "Glibc unpacking failed"
+      crosser_error "$LIBCNAME unpacking failed"
       exit 1
     fi
 
-    log_write 1 "Installing initial glibc headers"
-    if ! build_glibc glibc glibc-$VERSION_GLIBC \
-           "--with-tls --enable-add-ons=ports --disable-sanity-checks --with-sysroot=$SYSPREFIX --with-headers=$SYSPREFIX/usr/include" \
-           "install-headers" "headers"
+    if ! (is_minimum_version $LIBCVER 2.8 ||
+          patch_src $LIBCDIR glibc_upstream_finc)                                 ||
+       ! (is_minimum_version $LIBCVER 2.8 ||
+          patch_src $LIBCDIR/$LIBCNAME-ports-$LIBCVER glibc_ports_arm_docargs)    ||
+       ! (is_minimum_version $LIBCVER 2.8 ||
+          patch_src $LIBCDIR/$LIBCNAME-ports-$LIBCVER glibc_ports_arm_pageh_inc)  ||
+       ! patch_src $LIBCDIR/$LIBCNAME-ports-$LIBCVER glibc_ports_arm_tlsinc       ||
+       ! (is_smaller_version $LIBCVER 2.9 ||
+          patch_src $LIBCDIR/$LIBCNAME-ports-$LIBCVER glibc_upstream_arm_sigsetjmp)
     then
-      log_error "Failed to install initial glibc headers"
+      crosser_error "$LIBCNAME patching failed"
+      exit 1
+    fi
+
+#    if ! autogen_component eglibc $VERSION_EGLIBC "autoconf"
+#    then
+#      crosser_error "Eglibc autogen failed"
+#      exit 1
+#    fi
+
+    log_write 1 "Installing initial $LIBCNAME headers"
+    if ! build_glibc $LIBCNAME $LIBCDIR \
+           "--with-tls --enable-add-ons=$LIBCNAME-ports-$LIBCVER --disable-sanity-checks --with-sysroot=$SYSPREFIX --with-headers=$SYSPREFIX/usr/include" \
+           "install-headers install-bootstrap-headers=yes" "headers"
+    then
+      log_error "Failed to install initial $LIBCNAME headers"
       exit 1
     fi
 
     if ! dummy_glibc_objects
     then
-      log_error "Failed to build dummy glibc objects"
+      log_error "Failed to build dummy $LIBCNAME objects"
       exit 1
     fi
 
@@ -905,11 +783,11 @@ then
       exit 1
     fi
 
-    if ! build_glibc glibc glibc-$VERSION_GLIBC \
-             "--with-tls --with-sysroot=$SYSPREFIX --with-headers=$SYSPREFIX/usr/include --enable-add-ons=ports,nptl" \
+    if ! build_glibc $LIBCNAME $LIBCDIR \
+             "--with-tls --with-sysroot=$SYSPREFIX --with-headers=$SYSPREFIX/usr/include --enable-add-ons=$LIBCNAME-ports-$LIBCVER,nptl" \
              "all install"
     then
-      crosser_error "Failed to build final glibc"
+      crosser_error "Failed to build final $LIBCNAME"
       exit 1
     fi
 
@@ -924,8 +802,7 @@ then
 
   fi
 
-  echo "Setup:   $TARGET"          >  "$PREFIX/crosser/crosser.hierarchy"
-  echo "Version: $CROSSER_VERSION" >> "$PREFIX/crosser/crosser.hierarchy"
+  write_crosser_env "$PREFIX" "$TARGET"
 
   generate_setup_scripts $PREFIX
 else # STEP_CHAIN
@@ -936,7 +813,7 @@ fi   # STEP_CHAIN
 
 export CCACHE_DIR="$PREFIX/.ccache"
 
-export PKG_CONFIG_PATH="$SYSPREFIX/lib/pkgconfig:$SYSPREFIX/usr/lib/pkgconfig"
+export PKG_CONFIG_LIBDIR="$SYSPREFIX/lib/pkgconfig:$SYSPREFIX/usr/lib/pkgconfig"
 
 if test "x$STEP_BASELIB" = "xyes"
 then
@@ -951,7 +828,7 @@ then
   fi
 
   # Some patches work only when compiling for Windows target and
-  # thus used in libstack.sh only
+  # thus are used in libstack.sh only
   if ! patch_src zlib               zlib_cctest               ||
      ! patch_src zlib               zlib_seeko
   then
@@ -959,9 +836,9 @@ then
     exit 1
   fi
 
-  if ! build_zlib                zlib   zlib                  \
-       "--shared"                                             ||
-     ! build_with_cross_compiler libpng libpng-$VERSION_PNG
+  if ! build_zlib                zlib    zlib                 \
+        "--shared"                                            ||
+     ! build_with_cross_compiler libpng  libpng-$VERSION_PNG
   then
     crosser_error "Baselib build failed"
     exit 1
@@ -985,12 +862,33 @@ then
            ( patch_src glib-$VERSION_GLIB glib_gmoddef &&
              autogen_component glib    $VERSION_GLIB "aclocal automake autoconf" ))      ||
        ! build_with_cross_compiler glib          glib-$VERSION_GLIB                      \
-         "--prefix=/usr $GLIB_VARS"                                                                    ||
-       ! unpack_component          freetype      $VERSION_FREETYPE                       ||
+         "--prefix=/usr $GLIB_VARS"
+    then
+      crosser_error "Glib build failed"
+      exit 1
+    fi
+
+    STEP="gtk(ft1)"
+    STEPADD=""
+    if ! unpack_component          freetype      $VERSION_FREETYPE                       ||
        ! build_with_cross_compiler freetype      freetype-$VERSION_FREETYPE              \
-         "--prefix=$PREFIX/interm" "" "/"                                                ||
-       ! build_with_cross_compiler freetype      freetype-$VERSION_FREETYPE              ||
-       ! unpack_component          expat         $VERSION_EXPAT                          ||
+         "--prefix=$PREFIX/interm" "" "/"
+    then
+      crosser_error "Intermediate freetype build failed"
+      exit 1
+    fi
+
+    STEP="gtk(ft2)"
+    STEPADD=""
+    if ! build_with_cross_compiler freetype      freetype-$VERSION_FREETYPE
+    then
+      crosser_error "Target freetype build failed"
+      exit 1
+    fi
+
+    STEP="gtk"
+    STEPADD="     "
+    if ! unpack_component          expat         $VERSION_EXPAT                          ||
        ! build_with_cross_compiler expat         expat-$VERSION_EXPAT                    ||
        ! unpack_component          pixman        $VERSION_PIXMAN                         ||
        ! build_with_cross_compiler pixman        pixman-$VERSION_PIXMAN                  \
@@ -1010,6 +908,40 @@ then
   fi
 fi
 
+if test "x$STEP_SDL" = "xyes"
+then
+  STEP="sdl"
+  STEPADD="     "
+
+  if test "x$LIBC_MODE" != "xglibc"
+  then
+    log_write 1 "Step sdl not available for $LIBC_MODE based builds, skipping"
+  elif test "x$KERN_ARCH" = "xarm"
+  then
+    log_write 1 "Step sdl not available for $KERN_ARCH architecture, skipping"
+  else
+
+    if ! kernel_setup full                                                ||
+       ! unpack_component          svgalib $VERSION_SVGALIB               ||
+       ! patch_src svgalib-$VERSION_SVGALIB svgalib_cfg                   ||
+       ! patch_src svgalib-$VERSION_SVGALIB svgalib_gentoo_k26            ||
+       ! patch_src svgalib-$VERSION_SVGALIB svgalib_gentoo_k2628          ||
+       ! patch_src svgalib-$VERSION_SVGALIB svgalib_arm_outsb             ||
+       ! patch_src svgalib-$VERSION_SVGALIB svgalib_nostrip               ||
+       ! patch_src svgalib-$VERSION_SVGALIB svgalib_glinc                 ||
+       ! build_svgalib             svgalib svgalib-$VERSION_SVGALIB       \
+          "clean install"                                                 ||
+       ! unpack_component          SDL       $VERSION_SDL                 ||
+       ! build_with_cross_compiler SDL       SDL-$VERSION_SDL             ||
+       ! unpack_component          SDL_image $VERSION_SDL_IMAGE           ||
+       ! build_with_cross_compiler SDL_image SDL_image-$VERSION_SDL_IMAGE
+    then
+      crosser_error "sdl stack build failed"
+      exit 1
+    fi
+  fi
+fi
+
 if test "x$STEP_TEST" = "xyes"
 then
   STEP="test"
