@@ -71,26 +71,25 @@ fi
 # $1 - Component
 # $2 - Version
 # $3 - Extra configure options
-build_component_srcdir()
+build_component()
 {
-  build_component_full "$1" "$1" "$2" "$3" "src"
+  build_component_full "$1" "$1" "$2" "$3"
 }
 
 # $1 - Component
 # $2 - Version
 # $3 - Extra configure options
-build_component()
+build_component_host()
 {
-  build_component_full "$1" "$1" "$2" "$3"
+  build_component_full "host-$1" "$1" "$2" "$3" "" "native"
 }
 
 # $1 - Build dir
 # $2 - Component
 # $3 - Version
 # $4 - Extra configure options
-# $5 - Build in src dir?
-# $6 - Overwrite libtool
-# $7 - Native
+# $5 - Overwrite libtool ('overwrite')
+# $6 - Native ('native')
 build_component_full()
 {
   log_packet "$1"
@@ -103,25 +102,16 @@ build_component_full()
     return 1
   fi
 
-  if test "x$5" != "xsrc" ; then
-    BUILDDIR="$MAINBUILDDIR/$1"
-    if ! mkdir -p "$BUILDDIR"
-    then
-      log_error "Failed to create directory $BUILDDIR"
-      return 1
-    fi
-    cd $BUILDDIR
-    SRCDIR="$MAINSRCDIR/$SUBDIR"
-  else
-    if ! cd $MAINSRCDIR/$SUBDIR
-    then
-      log_error "Cannot change to directory $MAINSRCDIR/$SUBDIR"
-      return 1
-    fi
-    SRCDIR="."
+  BUILDDIR="$MAINBUILDDIR/$1"
+  if ! mkdir -p "$BUILDDIR"
+  then
+    log_error "Failed to create directory $BUILDDIR"
+    return 1
   fi
+  cd $BUILDDIR
+  SRCDIR="$MAINSRCDIR/$SUBDIR"
 
-  if test "x$7" != "xnative"
+  if test "x$6" != "xnative"
   then
     CONFOPTIONS="--prefix=$PREFIX --build=$BUILD --host=$TARGET --target=$TARGET $4"
     export CPPFLAGS="-isystem $PREFIX/include -isystem $TGT_HEADERS $TGT_MARCH $USER_CPPFLAGS"
@@ -132,7 +122,7 @@ build_component_full()
     unset LDFLAGS
   fi
 
-  log_write 1 "Configuring: $1"
+  log_write 1 "Configuring $1"
   log_write 3 "  Options: \"$CONFOPTIONS\""
   log_flags
 
@@ -142,14 +132,14 @@ build_component_full()
     return 1
   fi
 
-  if test "x$6" = "xoverwrite" ; then
+  if test "x$5" = "xoverwrite" ; then
     log_write 2 "Copying working libtool to $1"
     if ! cp $MAINBUILDDIR/libtool/libtool .
     then
       log_error "Failed to copy libtool"
       return 1
     fi
-  elif test "x$6" != "x" ; then
+  elif test "x$5" != "x" ; then
     log_error "Illegal libtool overwrite parameter $6"
     return 1
   fi
@@ -200,7 +190,7 @@ build_zlib()
   # TODO: zlib build doesn't like this variable, check why.
   unset TARGET_ARCH
 
-  log_write 1 "Configuring: $1"
+  log_write 1 "Configuring $1"
   log_write 3 "  Options: \"$CONFOPTIONS\""
   log_flags
 
@@ -277,8 +267,8 @@ update_aux_files() {
 
 cd $(dirname $0)
 
-if ! source $MAINSETUPDIR/native.sh ; then
-  log_error "Failed to read $MAINSETUPDIR/native.sh"
+if ! . $MAINDIR/setups/native.sh ; then
+  log_error "Failed to read $MAINDIR/setups/native.sh"
   exit 1
 fi
 NATIVE_ARCH="$TMP_ARCH"
@@ -287,11 +277,11 @@ BUILD="$NATIVE_ARCH-$NATIVE_OS"
 
 SETUP="win32"
 
-if ! test -e "$MAINSETUPDIR/$SETUP.conf" ; then
+if ! test -e "$MAINDIR/setups/$SETUP.conf" ; then
   log_error "Can't find setup \"$SETUP.conf\""
   exit 1
 fi
-source "$MAINSETUPDIR/$SETUP.conf"
+source "$MAINDIR/setups/$SETUP.conf"
 
 if test "x$LIBSTACK" = "xno"
 then
@@ -315,8 +305,9 @@ TGT_MARCH="-march=$TARGET_ARCH"
 
 export LIBC_MODE="none"
 
-export PREFIX=$(setup_prefix_default "$(pwd)/install-<TARGET>-<DATE>" "$PREFIX")
-export NATIVE_PREFIX=$(setup_prefix_default "/usr/local/crosser/$CROSSER_VERSION/lshost" "$LSHOST_PREFIX")
+export PREFIX=$(setup_prefix_default "$HOME/.crosser/<VERSION>/winstack" "$PREFIX")
+export NATIVE_PREFIX=$(setup_prefix_default "$HOME/.crosser/$CROSSER_VERSION/lshost" \
+                       "$LSHOST_PREFIX")
 
 export USER_CPPFLGS="$CPPFLAGS"
 export USER_LDFLAGS="$LDFLAGS"
@@ -372,14 +363,14 @@ fi
 
 if test "x$CROSSER_DOWNLOAD" = "xyes"
 then
-  if ! (cd $PACKETDIR && $MAINDIR/scripts/download_packets.sh "win,sdl" )
+  if ! (cd $PACKETDIR && $MAINDIR/scripts/download_packets.sh "win" )
   then
     log_error "Downloading packets failed"
     exit 1
   fi
 fi
 
-export PKG_CONFIG_PATH=$PREFIX/lib/pkgconfig
+export PKG_CONFIG_LIBDIR=$PREFIX/lib/pkgconfig
 
 BASEVER_LIBTOOL="$(basever_libtool $VERSION_LIBTOOL)"
 
@@ -391,7 +382,7 @@ if ! unpack_component  glib       $VERSION_GLIB             ||
         patch_src glib-$VERSION_GLIB glib_acsizeof   &&
         autogen_component glib       $VERSION_GLIB   \
          "libtoolize aclocal automake autoconf" ))          ||
-   ! build_component_full host_glib glib $VERSION_GLIB "" "" "" native
+   ! build_component_host glib $VERSION_GLIB
 then
   log_error "Native build failed"
   exit 1
@@ -426,7 +417,7 @@ then
      ! patch_src        libjpeg6b  jpeg_ar                  ||
      ! patch_src        libjpeg6b  jpeg_noundef             ||
      ! update_aux_files     libjpeg6b  $VERSION_JPEG        ||
-     ! build_component_full libjpeg6b libjpeg6b  $VERSION_JPEG "--enable-shared" "" "overwrite"
+     ! build_component_full libjpeg6b libjpeg6b  $VERSION_JPEG "--enable-shared" "overwrite"
   then
     log_error "Libjpeg build failed"
     exit 1
@@ -467,7 +458,7 @@ fi
 
 if ! autogen_component tiff       $VERSION_TIFF                   ||
    ! build_component_full                                         \
-     tiff tiff $VERSION_TIFF "$CONF_JPEG_TIFF" "" "overwrite"     ||
+     tiff tiff $VERSION_TIFF "$CONF_JPEG_TIFF" "overwrite"        ||
    ! unpack_component  expat      $VERSION_EXPAT                  ||
    ! build_component   expat      $VERSION_EXPAT
 then
@@ -518,6 +509,7 @@ if ! unpack_component  gtk+       $VERSION_GTK                    ||
    ! ( is_minimum_version $VERSION_GTK      2.13.2 ||
        patch_src gtk+-$VERSION_GTK          gtk_check_cxx )       ||
    ! ( is_smaller_version $VERSION_GTK      2.14.0 ||
+       is_minimum_version $VERSION_GTK      2.16.0 ||
        patch_src gtk+-$VERSION_GTK          gtk_gailutildef )     ||
    ! autogen_component gtk+       $VERSION_GTK                    \
      "libtoolize aclocal automake autoconf"                       ||
@@ -540,6 +532,11 @@ if test "x$AUTOWINE" = "xyes" ; then
     exit 1
   fi
 fi
+log_write 1 "Creating setup.bat"
+(
+  echo -n -e "bin\pango-querymodules.exe > etc\pango\pango.modules\r\n"
+  echo -n -e "bin\gdk-pixbuf-query-loaders.exe > etc\gtk-2.0\gdk-pixbuf.loaders\r\n"
+) > $PREFIX/setup.bat
 log_write 1 "IMPORTANT: Remember to create configuration files when installing to target"
 
 if ! unpack_component  SDL        $VERSION_SDL          ||
